package org.seasar.struts.extension.action;

import java.io.IOException;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.config.ForwardConfig;
import org.seasar.framework.beans.BeanDesc;
import org.seasar.framework.beans.PropertyDesc;
import org.seasar.framework.beans.factory.BeanDescFactory;
import org.seasar.struts.action.S2RequestProcessor;
import org.seasar.struts.action.WrapperUtil;
import org.seasar.struts.config.S2ActionMapping;
import org.seasar.struts.config.S2ExecuteConfig;
import org.seasar.struts.extension.annotation.ReadOnly;
import org.seasar.struts.extension.annotation.SessionScope;
import org.seasar.struts.extension.util.SerializeUtilz;
import org.seasar.struts.util.S2ActionMappingUtil;
import org.seasar.struts.util.S2ExecuteConfigUtil;

/**
 * {@link S2RequestProcessor} の拡張クラスです。
 *
 * @author awaawa
 *
 */
public class XRequestProcessor extends S2RequestProcessor {

	/** ログ。 */
	private static final Log LOG
			= LogFactory.getLog(XRequestProcessor.class);

	/** actionForm 初期化マーク。 */
	private static final String ACTION_FORM_INIT
			= "org.seasar.struts.extension.actionForm.init";

	/**
	 * {@inheritDoc}
	 * {@link SessionScope} が設定されているフィールドを session に保存します。
	 */
	@Override
	protected void processForwardConfig(
			HttpServletRequest request,
			HttpServletResponse response,
			ForwardConfig forward) throws IOException, ServletException {

		this.exportPropertiesToSession(
				request,
				S2ActionMappingUtil.getActionMapping(),
				S2ExecuteConfigUtil.getExecuteConfig());

		super.processForwardConfig(request, response, forward);
	}

	/**
	 * {@link SessionScope} が設定されている プロパティをセッションに設定します。
	 * @param request リクエスト
	 * @param actionMapping アクションマッピング
	 * @param executeConfig 実行設定
	 */
	protected void exportPropertiesToSession(
			HttpServletRequest request,
			S2ActionMapping actionMapping,
			S2ExecuteConfig executeConfig) {

		BeanDesc actionBeanDesc = actionMapping.getActionBeanDesc();
		for (int i = 0; i < actionBeanDesc.getPropertyDescSize(); i++) {
			Object action = actionMapping.getAction();
			PropertyDesc pd = actionBeanDesc.getPropertyDesc(i);
			if (pd.isReadable()
					&& isExportableProperty(pd)
					&& pd.getField() != null
					&& pd.getField().getAnnotation(
							SessionScope.class) != null) {

				Object value = pd.getValue(action);

				if (value != null) {
					HttpSession session = request.getSession();
					session.setAttribute(pd.getPropertyName(), value);
				}
			}
		}
	}

	/**
	 * {@inheritDoc}
	 * ただし、Action に ActionForm がない場合、設定しません。
	 * また、{@link ReadOnly} が設定されているフィールドは設定しません。
	 */
	@Override
	protected void setProperty(Object bean, String name, Object value) {
		this.setProperty(bean, name, value, true);
	}

	/**
	 * プロパティを設定します。
	 * ただし、Action に ActionForm がない場合、設定しません。
	 * また、{@link ReadOnly} が設定されているフィールドは設定しません。
	 * @param bean ActionForm
	 * @param name プロパティ名
	 * @param value プロパティの値
	 * @param readOnlyCheck 読み取り専用チェックをする場合、{@code true}
	 */
	protected void setProperty(
			Object bean,
			String name,
			Object value,
			boolean readOnlyCheck) {

		Object action = S2ActionMappingUtil.getActionMapping().getAction();

		if (bean == null || bean.equals(action)) {
			return;
		}

		BeanDesc beanDesc = BeanDescFactory.getBeanDesc(bean.getClass());
		if (readOnlyCheck
				&& beanDesc.hasField(name)
				&& beanDesc.getField(name).getAnnotation(ReadOnly.class) != null) {
			return;
		}

		super.setProperty(bean, name, value);
	}

	/**
	 * {@inheritDoc}
	 * リクエストパラメータに Base64 エンコードされた ActionForm の直列化情報がある場合、その情報を設定します。
	 */
	@Override
	protected ActionForm processActionForm(
			HttpServletRequest request,
			HttpServletResponse response,
			ActionMapping mapping) {

		if (!Boolean.TRUE.equals(request.getAttribute(ACTION_FORM_INIT))) {

			request.setAttribute(ACTION_FORM_INIT, Boolean.TRUE);

			String name = S2ActionMappingUtil.getActionMapping().getAttribute();
			String str = request.getParameter(name);

			Object obj = null;
			try {
				obj = SerializeUtilz.fromStringToObject(str);
			} catch (RuntimeException e) {
				LOG.warn("ActionForm のデシリアイズに失敗。", e);
			}

			if (obj != null) {

				BeanDesc beanDesc = BeanDescFactory.getBeanDesc(obj.getClass());

				for (int i = 0; i < beanDesc.getPropertyDescSize(); i++) {
					PropertyDesc propertyDesc = beanDesc.getPropertyDesc(i);

					if (propertyDesc.isWritable()) {
						setProperty(
							S2ActionMappingUtil.getActionMapping().getActionForm(),
							propertyDesc.getPropertyName(),
							propertyDesc.getValue(obj),
							false);
					}
				}
			}
		}

		ActionForm actionForm = super.processActionForm(
											request,
											response,
											mapping);

		return actionForm;
	}
}
